package de.k621.avr.gps.converter;

import java.io.BufferedReader;
import java.io.BufferedWriter;

/**
 * Die Allgemeine Konvertierer-Klasse. Erwartet einige allg. benötigte
 * Objekte. Außerdem besitzt die Klasse eine Methode zur Verifizierung
 * von NMEA Kommandos.
 *
 * @author Martin Matysiak
 */
public class KConverter extends Thread {

  /**
   * Die Eingabedatei
   */
  protected BufferedReader fInput = null;
  /**
   * Die Ausgabedatei
   */
  protected BufferedWriter fOutput = null;
  /**
   * Das Statusinterface
   */
  protected KProgressViewer fInterface = null;

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   */
  public KConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface) {
    fInput = pInput;
    fOutput = pOutput;
    fInterface = pInterface;

    fInterface.setBarMax(1);
    fInterface.updateBar(0);
    fInterface.updateText("Thread initalisiert");

    this.start();
  }

  /**
   * Rechnet die Prüfsumme der gegebenen NMEA Zeile aus und vergleicht sie
   * mit der gesendeten
   *
   * @param line Das NMEA-Kommando
   * @return true wenn Kommando gültig, ansonsten false
   */
  protected static boolean isValidMsg(String line) {
    byte checksum = 0;
    int i = 1;
    boolean end = false;

    for (i = 1; (i < line.length()) && (!end); i++) {
      if (line.charAt(i) != '*') {
        checksum ^= line.charAt(i);
      } else {
        end = true;
      }
    }

    byte givenSum = 0;

    try {
      givenSum = Byte.parseByte(line.substring(i), 16);
    } catch (Exception ex) {
      givenSum = 0;
    }

    if (checksum != givenSum) {
      return false;
    } else {
      return true;
    }
  }

  @Override
  public void run() {
    fInterface.updateText("FEHLER :: Instanz der KConverter Klasse erstellt.");
  }
}
