package de.k621.avr.gps.converter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Vector;

/**
 * Die Konvertierungsklasse für Höhendiagramme.
 *
 * @author Martin Matysiak
 */
public class KElevationConverter extends KConverter {

  private int fMinimumSattelites = 0;
  private float fMaximumHDOP = Float.POSITIVE_INFINITY;

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   */
  public KElevationConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface) {
    super(pInput, pOutput, pInterface);
  }

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   * @param pMinSattelites Die minimale Anzahl an Satelliten, die eine Nachricht besitzen muss
   * @param pMaxHDOP Der maximale HDOP Wert, den eine Nachricht besitzen darf
   */
  public KElevationConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface, int pMinSattelites, float pMaxHDOP) {

    super(pInput, pOutput, pInterface);

    fMinimumSattelites = pMinSattelites;
    fMaximumHDOP = pMaxHDOP;

  }

  @Override
  public void run() {
    fInterface.updateText("Bereite Konvertierung vor...");
    fInterface.setBarMax(1); //1 Schritt: Einlesen & Schreiben

    try {
      String inLine = fInput.readLine();
      float minAlt = Float.POSITIVE_INFINITY, maxAlt = 0;
      int counter = 0;

      while (inLine != null && !inLine.isEmpty()) {

        //Nicht GGA-Meldungen ignorieren
        if (inLine.startsWith("$GPGGA") && isValidMsg(inLine)) {
          KGGAMessage message = new KGGAMessage(inLine);

          if (message.isValid() &&
                  message.hasSattelites(fMinimumSattelites) &&
                  message.hasLowerHdopThan(fMaximumHDOP)) {

            minAlt = message.getAltitude() < minAlt ? message.getAltitude() : minAlt;
            maxAlt = message.getAltitude() > maxAlt ? message.getAltitude() : maxAlt;

            fOutput.write(/*message.getTime(KGGAMessage.RAW).replace('.', ',')*/counter++ + ";" + Float.toString(message.getAltitude()).replace('.', ',') + "\n");

          }

        }

        inLine = fInput.readLine();
      }

    } catch (IOException ioex) {
      javax.swing.JOptionPane.showMessageDialog(null,
              ioex.getLocalizedMessage(),
              "Fehler",
              javax.swing.JOptionPane.ERROR_MESSAGE);
    } finally {
      try {
        fInput.close();
        fOutput.close();
      } catch (IOException ioex) {
        //Nichts tun
      }
    }

    fInterface.updateText("Datei erfolgreich erstellt!");
    fInterface.updateBar(-1);
  }
}
