package de.k621.avr.gps.converter;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;

/**
 * Die Konvertierungsklasse für Geschwindigkeitsdiagramme.
 *
 * @author Martin Matysiak
 */
public class KSpeedConverter extends KConverter {

  /**
   * Konstruktor der Klasse
   *
   * @param pInput Die Eingabedatei
   * @param pOutput Die Ausgabedatei
   * @param pInterface Das Statusinterface
   */
  public KSpeedConverter(BufferedReader pInput, BufferedWriter pOutput,
          KProgressViewer pInterface) {
    super(pInput, pOutput, pInterface);
  }

  @Override
  public void run() {
    fInterface.updateText("Bereite Konvertierung vor...");
    fInterface.setBarMax(1); //1 Schritte: Einlesen & Schreiben gleichzeitig

    try {
      String inLine = fInput.readLine();

      int counter = 0;

      while (inLine != null && !inLine.isEmpty()) {

        //Nicht GGA-Meldungen ignorieren
        if (inLine.startsWith("$GPVTG") && isValidMsg(inLine)) {
          KVTGMessage message = new KVTGMessage(inLine);

          if (message.isValid()) {
            fOutput.write(counter++ + ";" + Float.toString(message.getSpeed()).replace('.', ',') + "\n");
          }

        }

        inLine = fInput.readLine();
      }

    } catch (IOException ioex) {
      javax.swing.JOptionPane.showMessageDialog(null,
              ioex.getLocalizedMessage(),
              "Fehler",
              javax.swing.JOptionPane.ERROR_MESSAGE);
    } finally {
      try {
        fInput.close();
        fOutput.close();
      } catch (IOException ioex) {
        //Nichts tun
      }
    }

    fInterface.updateText("Datei erfolgreich erstellt!");
    fInterface.updateBar(1);
  }
}
